local ldb = LibStub:GetLibrary("LibDataBroker-1.1")

local addon = LibStub("AceAddon-3.0"):NewAddon("Broker2FuBar", "AceConsole-3.0", "AceHook-3.0")

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("Broker2FuBarDB", {
		profile = {
			objects = {
				['*'] = true,
			},
		},
	})

	LibStub("AceConfig-3.0"):RegisterOptionsTable(self.name, self.options)
			
	self.objectOptionsMetaTable = { __index = self.objectOptionsTemplate }
	
	self.fubared = {}

	for name, data_object in ldb:DataObjectIterator() do
		self:FuBarize(name, data_object)
	end

	ldb.RegisterCallback(self, "LibDataBroker_DataObjectCreated")
end

function addon:LibDataBroker_DataObjectCreated(event, name, data_object)
	self:FuBarize(name, data_object)
end

function addon:FuBarize(name, data_object)
	if not self.db.profile.objects[name] then return end
	if self.fubared[name] then return end
	
	local fu = LibStub("AceAddon-3.0"):NewAddon(name .. "_B2Fu", "LibFuBarPlugin-Mod-3.0")
	fu.data_object = data_object
	fu.db = self.db:RegisterNamespace(name, {profile={}})

	fu:SetFuBarOption('tooltipType', data_object.tooltip and 'Custom' or 'GameTooltip')
	fu:SetFuBarOption('configType', 'AceConfigDialog-3.0')
	fu:SetFuBarOption('aceConfig30', self.name)
	
	self.objectOptions[name] = setmetatable({name = name, handler = fu}, self.objectOptionsMetaTable)
	
	function fu:OnUpdateFuBarText()
		self:SetFuBarText(data_object.text)
	end
	function fu:OnUpdateFuBarIcon()
		-- this isn't an official fubar method, note
		self:SetFuBarOption('iconPath', data_object.icon)
	end
	function fu:OnFuBarClick(button)
		if data_object.OnClick then
			data_object.OnClick(self:GetPanel(), button)
		end
	end
	function fu:OnFuBarEnter()
		if data_object.OnEnter then
			data_object.OnEnter(self:GetPanel())
		end
	end
	function fu:OnFuBarLeave(button)
		if data_object.OnLeave then
			data_object.OnLeave(self:GetPanel())
		end
	end
	function fu:OnUpdateFuBarTooltip()
		if data_object.tooltip then
			if data_object.tooltiptext then
				data_object.tooltip:SetText(data_object.tooltiptext)
			end
			data_object.tooltip:Show()
		elseif data_object.OnTooltipShow then
			data_object.OnTooltipShow(GameTooltip)
		elseif data_object.tooltiptext then
			GameTooltip:SetText(data_object.tooltiptext)
		end
	end
	
	ldb.RegisterCallback(fu, "LibDataBroker_AttributeChanged_"..name.."_text", "OnUpdateFuBarText")
	ldb.RegisterCallback(fu, "LibDataBroker_AttributeChanged_"..name.."_icon", "OnUpdateFuBarIcon")
	ldb.RegisterCallback(fu, "LibDataBroker_AttributeChanged_"..name.."_tooltiptext", "OnUpdateFuBarTooltip")
	
	fu:OnUpdateFuBarIcon()
	
	self.fubared[name] = fu
end

